
#include <unistd.h>

#define _ENABLE_TILT
#include <wiimote.h>

#include <vdr/tools.h>
#include <vdr/plugin.h>

#include "bewegung_reader.h"
#include "bewegung_setup.h"
#include "bewegung_config.h"
#include "bewegung_analyse.h"


cBewegungReaderThread *cBewegungReaderThread::m_Instance = NULL;
t_motionValues cBewegungReaderThread::motionValues = {false,false,false,false,false,false,false,false,false,false,false,false,false};

wiimote_t cBewegungReaderThread::wiimote = WIIMOTE_INIT;

cBewegungReaderThread::cBewegungReaderThread()
   : cThread("WiiVDR: wiireader")
{
   m_Active = false;
   m_lastUpdate = time(NULL);
   isyslog("[wiivdr] starting reader-thread\n");
}

cBewegungReaderThread::~cBewegungReaderThread() 
{
   if (m_Active)
      Stop();
}

void cBewegungReaderThread::Init() 
{
   if (m_Instance == NULL) 
   {
      m_Instance = new cBewegungReaderThread();
      m_Instance->Start();
   }
}

void cBewegungReaderThread::Exit(void) 
{
   if (m_Instance != NULL) 
   {
      m_Instance->Stop();
      DELETENULL(m_Instance);
   }
}

void cBewegungReaderThread::Stop(void) 
{
   m_Active = false;
   Cancel(6);
}

cBewegungReaderThread *cBewegungReaderThread::getInstance(void)
{
   return m_Instance;
}

int cBewegungReaderThread::Connect(void)
{ 
   int rc = 0;
   //wiimote_t wiimotes[4];

   if (bewegungConfig.enableConnect)
   {
      rc = wiimote_connect(&wiimote, bewegungConfig.macAddress);

      // nmotes = wiimote_discover(wiimotes, 1);

      if (rc < 0) 
      {
         // Because the plugin searches very few seconds for a wiimote, 
         // this output would fill the syslog very fast.

         //esyslog("[wiivdr] error: cWiiReaderThread::Connect: no wiimote found!(rc=%d)",rc);
      }

      else
      {
         Skins.QueueMessage(mtInfo, tr("Found wiimote!"),1,1);
         isyslog("[wiivdr] found wiimote: %s\n", wiimote.link.r_addr);
    
         cCondWait::SleepMs(50);

         wiimote.led.bits  = 0;    // disable all current leds
         wiimote.led.four  = 1;    // enable forth led
         wiimote.mode.acc  = 0;    // disable accelerometer
         wiimote.mode.ir   = 0;    // disable ir-sensor
         wiimote.mode.ext  = 0;    // disable nunchuk etc

         wiimote_update(&wiimote);
      }
   }

   return rc;
}

int cBewegungReaderThread::Disconnect(void)
{
   wiimote.keys.home = false;
   wiimote.led.bits = 0;                        // disable all current leds
   Skins.QueueMessage(mtInfo, tr("Disconnecting wiimote!"),1,1);
   isyslog("[wiivdr] Disconnecting wiimote: %s\n", wiimote.link.r_addr);
   wiimote_disconnect(&wiimote);
   
   wiimote.link.status = WIIMOTE_STATUS_DISCONNECTED;

   return 0;
}

void cBewegungReaderThread::Action(void)
{
   uint64_t lastX = cTimeMs::Now();
   uint64_t lastY = cTimeMs::Now();

   m_Active = true;

   while (m_Active)
   {
      while (wiimote_is_open(&wiimote)) 
      {
         if ((wiimote_update(&wiimote) < 0))
         {
            // disconnect due to error

            Disconnect();
         }
         else if (wiimote.keys.home) 
         {
            // disconnected on user request

            Disconnect();
         }
         else
         {
            // dsyslog("[wiivdr] loop");
	
            // set the flags

            if (wiimote.keys.a == 1)
               motionValues.doPressOK = true;

            if (wiimote.keys.one == 1)
               motionValues.doPressMenu = true;

            if (wiimote.keys.two == 1)
               motionValues.doPressBack = true;

            if (wiimote.keys.up == 1)
               motionValues.doPressRed = true;

            if (wiimote.keys.down == 1)
               motionValues.doPressYellow = true;

            if (wiimote.keys.right == 1)
               motionValues.doPressGreen = true;

            if (wiimote.keys.left == 1)
               motionValues.doPressBlue = true;

            if (wiimote.keys.plus == 1)
               motionValues.doPressUp = true;
            
            if (wiimote.keys.minus == 1)
               motionValues.doPressDown = true;

            if (wiimote.keys.b == 1)
            {
               // x = turn left/right
               // y = pitch up/down
               // z = pitch left/rigth

               wiimote.mode.acc = 1;                           // enable accelerometer

               // Angels of -90/+90 seem to be used for "unshure" or something like this, ignore it.
               // It also is possible to have a NAN in the values, because NAN!=NAN we have to 
               // compare the value with itself.

               if (wiimote.tilt.y == wiimote.tilt.y 
                   && abs(wiimote.tilt.y) != 90
                   && abs(wiimote.tilt.y) > bewegungConfig.tiltUpDown)
               {
                  // we have a valid and changed value

                  int delay = 500 / ((abs(wiimote.tilt.y)-bewegungConfig.tiltUpDown)/2 +1);

                  // dsyslog("[wiivdr], delay set to %d due to a y-angel of %f", 
                  //         delay, wiimote.tilt.y);

                  if (lastY < cTimeMs::Now()-delay)
                  {
                     lastY = cTimeMs::Now();

                     if (wiimote.tilt.y > 0)
                        motionValues.doPressVolumeDown = true;
                     else 
                        motionValues.doPressVolumeUp = true;
                  }
               }

               if (wiimote.tilt.x == wiimote.tilt.x
                   && abs(wiimote.tilt.x) != 90
                   && abs(wiimote.tilt.x) > bewegungConfig.tiltLeftRight)
               {
                  // we have a valid and changed value

                  int delay = 500 / ((abs(wiimote.tilt.x)-bewegungConfig.tiltLeftRight)/5 +1);

                  // dsyslog("[wiivdr], delay set to %d due to a x-angel of %f", 
                  //         delay, wiimote.tilt.x);

                  if (lastX < cTimeMs::Now()-delay)
                  {
                     lastX = cTimeMs::Now();

                     if (wiimote.tilt.x > 0)
                        motionValues.doPressRight = true;
                     else
                        motionValues.doPressLeft = true;
                  }
               }
            }
            else
            {
               // b not pressed

               wiimote.mode.acc = 0;       // disable accelerometer
            }

            if (motionValues.doPressOK
                || motionValues.doPressUp
                || motionValues.doPressDown
                || motionValues.doPressLeft
                || motionValues.doPressRight
                || motionValues.doPressBlue
                || motionValues.doPressGreen
                || motionValues.doPressRed
                || motionValues.doPressYellow
                || motionValues.doPressVolumeUp
                || motionValues.doPressVolumeDown
                || motionValues.doPressMenu
                || motionValues.doPressBack)
            {
               dsyslog("[wiivdr] broadcast");
               cBewegungAnalyseThread::update.Broadcast();
            }
         }
      }

      Connect();
   };

   Disconnect();

   m_Instance = NULL;
}
